/*
 * mdBlinky.h
 * 
 * An Arduino library for blinking a LED.
 * consecutive button presses made in quick succession or if the 
 * button was held down for a long time.
 * 
 * Version 0.1, 2018-05-24.
 * 
 * by Michel Deslierres 
 *
 * See the accompanying file license.txt for the
 * conditions of use.
 * 
 */

#ifndef mdBlinky_h
#define mdBlinky_h

#include "Arduino.h"

/*
 * The LED is blinked a number of times consecutively at regular intervals:
 * 
 *                        +-------------1 cycle-----------+
 *                        |                               | 
 *                        +----blinks_----+               |
 *                        |               ||<--offTime_-->|
 *                        |               ||              |
 * X----XXX---------------XXX----XXX----XXX----------------XXX---XXX---XXX
 *                        | |       |  | 
 *                        | |    -->|  |<--blinkOffTime_
 *                        | |       
 *                     -->| |<- blinkOnTime_
 *                       
 */ 

#define DEFAULT_BLINKS                   2
#define DEFAULT_BLINK_OFF_TIME         200
#define DEFAULT_BLINK_ON_TIME           30
#define DEFAULT_OFF_TIME              1740
#define DEFAULT_CYCLES                   0

enum ledActive_t { ACTIVE_LOW, ACTIVE_HIGH };
enum blinkMode_t { OFF, ON, BLINK };

class mdBlinky
{
  public:
    // Create the object specifying the pin connected to the LED and 
    // if the LED is activated (turned on) with a LOW or HIGH value
    mdBlinky(uint8_t pin, ledActive_t ledActive); //constructor
  
    // Set attributes

    // A value of OFF turns the LED off, of ON turns the LED on and
    // BLINK starts the blink mode.
    void setMode(blinkMode_t value);

    // Number of blinks (on periods) in each cycle
    void setBlinks(uint16_t value);

    // Time in microseconds (ms) the led is on when blinked 
    void setBlinkOffTime(uint16_t value);

    // Time (ms) the led is off between two blinks
    void setBlinkOnTime(uint16_t value);

    // Time (ms) the led is off when not blinking
    void setOffTime(uint16_t value);

    // Number of cycles to run until the LED is turned off and the mode is then set off. 
    // If value == 0, the blinking goes on indefinitely (this is the default).
    void setCycles(uint16_t value);

    // Set all blink parameters at once. This changes the mode to BLINK.
    void setParams(uint16_t blinks, uint16_t blinkOffTime, uint16_t blinkOnTime, uint16_t offTime, uint16_t cycles = 0);

    // Select a blink pattern in a linear array of uint16_t logically arranged 
    // into 4 columns and as many rows as there are sets of parameters. 
    // Ex.
    // uint16_t params_array[] = { 2, 200, 30, 1740,  // heartbeat
    //                             1,   0, 50,   50,  // panic
    //                             0,   0,  0,    1,  // always on
    //                             0,   0,  0,    0,  // always off
    // Call with blinky.setParams(params_array, 1) to get the panic blinking
    // pattern. The last two entries are visually equivalent to setMode(ON)
    // and setMode(OFF) but make it easier to  create an array of all 
    // LED flashing patterns to be used a sketch to select the current 
    // pattern on the fly with an index.
    //
    // This changes the mode to BLINK.
    void setParams(uint16_t params_array[], uint8_t index = 0, uint16_t cycles = 0);
    
    // Get attribute
    blinkMode_t getMode(void);
    
    // Pump to update LED, return current state of LED
    // false = LED is off, true = LED is on.
    // MUST BE CALLED on a regular basis (once in every 
    // loop() iteration for example).
    boolean blink();
  
  private:
    uint8_t pin_;    
    uint8_t led_off_;                   
    uint8_t led_on_; 
    blinkMode_t blinkMode_ = OFF;                  
    uint16_t blinks_       = DEFAULT_BLINKS; 
    uint16_t blinkOffTime_ = DEFAULT_BLINK_OFF_TIME;
    uint16_t blinkOnTime_  = DEFAULT_BLINK_ON_TIME;
    uint16_t offTime_      = DEFAULT_OFF_TIME;
    uint16_t cycles_       = DEFAULT_CYCLES;
    
  
    // State variables 
    enum blinkState_t { LED_OFF, LED_ON, BLINK_ON, BLINK_OFF, OFF_TIME };
    blinkState_t blinkState_;
    unsigned long eventTime_; 
    uint16_t blinkCounter_;
   uint16_t cycleCounter_; 
 
};

#endif

